/*
    Copyright (C) HWPORT.COM
    All rights reserved.
    Author: JAEHYUK CHO <mailto:minzkn@minzkn.com>
*/

#if !defined(__def_source_main_c__)
# define __def_source_main_c__ "main.c"

/* ---- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define multicall_likely(m_expression) __builtin_expect((long)(!!(m_expression)),1L)
#define multicall_unlikely(m_expression) __builtin_expect((long)(!!(m_expression)),0L)

static int test1_main(int s_argc, char **s_argv);
static int test2_main(int s_argc, char **s_argv);

int main(int s_argc, char **s_argv);

static int test1_main(int s_argc, char **s_argv)
{
    (void)s_argc;
    (void)s_argv;

    (void)fprintf(stdout, "called %s\n", __func__);

    return(EXIT_SUCCESS);
}

static int test2_main(int s_argc, char **s_argv)
{
    (void)s_argc;
    (void)s_argv;

    (void)fprintf(stdout, "called %s\n", __func__);

    return(EXIT_SUCCESS);
}

int main(int s_argc, char **s_argv)
{
    static const struct {
        const char *m_name;
        int (*m_main)(int, char **);
    } cg_multicall_table[] = {
        {"test1", &test1_main},
        {"test2", &test2_main},
        {NULL, NULL}
    };

    int s_multicall_index;
    int s_shift = 0;

    if((s_argc >= 2) && (s_argv[1] != NULL)) {
        for(s_multicall_index = 0;cg_multicall_table[s_multicall_index].m_name != NULL;s_multicall_index++) {
            if(strcasecmp(s_argv[1], cg_multicall_table[s_multicall_index].m_name) == 0) {
                ++s_shift;
                break;
            }
        }
    }
    if(s_shift <= 0) {
        const char *c_program_name = strrchr(s_argv[0], '/');

        c_program_name = (c_program_name == NULL) ? s_argv[0] : &c_program_name[1];

        for(s_multicall_index = 0;cg_multicall_table[s_multicall_index].m_name != NULL;s_multicall_index++) {
            if(strcasecmp(c_program_name, cg_multicall_table[s_multicall_index].m_name) == 0) {
                break;
            }
        }
    }

    if(multicall_unlikely(cg_multicall_table[s_multicall_index].m_main == NULL)) { /* not defined multicall */
        (void)fprintf(
            stderr,
            "\n"
            "%s v0.0.1-0 (%s %s)\n"
            "Copyrights (C) MINZKN.COM - All rights reserved.\n"
            "\n"
            "usage: %s <function> [<options>]\n"
            "\n"
            "Currently defined functions(multi-call binary):\n",
            s_argv[0],
            __DATE__,
            __TIME__,
            s_argv[0]
        );
        for(s_multicall_index = 0;cg_multicall_table[s_multicall_index].m_name != NULL;s_multicall_index++) {
            (void)fprintf(
                stderr,
                "%s%s",
                (s_multicall_index == 0) ? "\t" : ", ",
                cg_multicall_table[s_multicall_index].m_name
            );
        }
        (void)fprintf(
            stderr,
            "\n\n"
        );

        return(EXIT_FAILURE);
    }

    return(cg_multicall_table[s_multicall_index].m_main(s_argc - s_shift, &s_argv[s_shift]));
}

/* ---- */

#endif

/* vim:set noexpandtab tabstop=4 shiftwidth=4 softtabstop=4 autoindent cindent smarttab fileencoding=utf8: */
/* End of source */
